@@ -251,6 +251,7 @@ public class DropboxFileAccess extends AbstractDropboxAccess implements Thumbnai
         }
         String path = toPath(source.getFolder(), source.getId());
         String destName = null != update && null != modifiedFields && modifiedFields.contains(Field.FILENAME) ? update.getFileName() : source.getId();
+        checkFolderExistence(destFolder);
         try {
             /*
              * ensure filename uniqueness in target folder
@@ -274,6 +275,7 @@ public class DropboxFileAccess extends AbstractDropboxAccess implements Thumbnai
 
     @Override
     public IDTuple move(IDTuple source, String destFolder, long sequenceNumber, File update, List<File.Field> modifiedFields) throws OXException {
+    	checkFolderExistence(destFolder);
         String path = toPath(source.getFolder(), source.getId());
         String destName = null != update && null != modifiedFields && modifiedFields.contains(Field.FILENAME) ? update.getFileName() : source.getId();
         String destPath = toPath(destFolder, destName);
@@ -322,11 +324,14 @@ public class DropboxFileAccess extends AbstractDropboxAccess implements Thumbnai
 
     @Override
     public IDTuple saveDocument(final File file, final InputStream data, final long sequenceNumber, final List<Field> modifiedFields) throws OXException {
+        boolean addVersion = null != modifiedFields && modifiedFields.contains(Field.VERSION_COMMENT);
         String path = FileStorageFileAccess.NEW == file.getId() ? null : toPath(file.getFolderId(), file.getId());
+        checkFolderExistence(file.getFolderId());
         try {
             final long fileSize = file.getFileSize();
             final long length = fileSize > 0 ? fileSize : -1L;
             Entry entry = null;
+            
             if (Strings.isEmpty(path) || !exists(file.getFolderId(), file.getId(), CURRENT_VERSION)) {
                 // Create
                 ThresholdFileHolder sink = null;
@@ -341,7 +346,11 @@ public class DropboxFileAccess extends AbstractDropboxAccess implements Thumbnai
                     boolean retry = true;
                     while (retry) {
                         try {
-                            entry = dropboxAPI.putFile(new StringBuilder(file.getFolderId()).append('/').append(fileName).toString(), sink.getStream(), length, null, null);
+                            if (addVersion) {
+                                entry = dropboxAPI.putFileOverwrite(new StringBuilder(file.getFolderId()).append('/').append(fileName).toString(), sink.getStream(), length, null);
+                            } else {
+                                entry = dropboxAPI.putFile(new StringBuilder(file.getFolderId()).append('/').append(fileName).toString(), sink.getStream(), length, null, null);
+                            }
                             retry = false;
                         } catch (DropboxServerException e) {
                             if (SC_CONFLICT != e.error) {
@@ -373,6 +382,22 @@ public class DropboxFileAccess extends AbstractDropboxAccess implements Thumbnai
             throw handle(e, path);
         }
     }
+    
+    
+    private void checkFolderExistence(String folderId) throws OXException{
+    	String folderPath = toId(folderId);
+        Entry folderEntry;
+		try {
+			folderEntry = dropboxAPI.metadata(folderPath, 1, null, false, CURRENT_VERSION);
+            if (folderEntry == null || folderEntry.isDeleted) {
+                throw FileStorageExceptionCodes.NO_SUCH_FOLDER.create();
+            }
+		} catch (DropboxException e) {
+			throw FileStorageExceptionCodes.PROTOCOL_ERROR.create(e, DropboxConstants.ID, e.getMessage());
+		}
+        
+    }
+
 
     @Override
     public void removeDocument(final String folderId, final long sequenceNumber) throws OXException {
